@@ -24,16 +24,12 @@ import yaml
 
 from tron.config import ConfigError
 from tron.config.schedule_parse import valid_schedule
-from tron.config.schedule_parse import ConfigConstantScheduler
-from tron.config.schedule_parse import ConfigIntervalScheduler
 from tron.utils.dicts import FrozenDict
+from tron.core.action import CLEANUP_ACTION_NAME
 
 
 log = logging.getLogger("tron.config")
 
-CLEANUP_ACTION_NAME = "cleanup"
-
-
 YAML_TAG_RE = re.compile(r'!\w+\b')
 
 def load_config(string_or_file):
@@ -563,6 +559,7 @@ class ValidateJob(ValidatorWithNamedPath):
         'all_nodes':            False,
         'cleanup_action':       None,
         'enabled':              True,
+        'queueing':             True
     }
 
     validators = {
@@ -577,15 +574,6 @@ class ValidateJob(ValidatorWithNamedPath):
         'enabled':              valid_bool,
     }
 
-    def set_defaults(self, job):
-        super(ValidateJob, self).set_defaults(job)
-        if 'queueing' not in job:
-            # Set queueing default based on scheduler. Usually False, But daily
-            # jobs probably deal with daily data and should not be skipped
-            job['queueing'] = not isinstance(job['schedule'],
-                    (ConfigConstantScheduler, ConfigIntervalScheduler)
-            )
-
     def _validate_dependencies(self, job, actions,
         base_action, current_action=None, stack=None):
         """Check for circular or misspelled dependencies."""
