@@ -127,7 +127,6 @@ class Job(Observable, Observer):
         self.action_graph   = job.action_graph
         self.enabled        = job.enabled
         self.output_path    = job.output_path
-        self.context        = job.context
         self.notify(self.EVENT_RECONFIGURED)
 
     @property
@@ -332,8 +331,10 @@ class JobScheduler(Observer):
         if queued_run:
             reactor.callLater(0, self.run_job, queued_run, run_queued=True)
 
-        if self.job.scheduler.schedule_on_complete:
-            self.schedule()
+        # Attempt to schedule a new run.  This will only schedule a run if the
+        # previous run was cancelled from a scheduled state, or if the job
+        # scheduler is `schedule_on_complete`.
+        self.schedule()
     handler = handle_job_events
 
     def get_runs_to_schedule(self, ignore_last_run_time=False):
